"""Mini Graphics

Module designed for use for introductory computer programming.

Wrapper around standard Tk module to enable old-school monochrome graphics.
Paint/unpaint "cells" on screen.

paint (as well as unpaint and clear) open the graphics window if it has not yet
been opened.

This version is nearly identical to simple_graphics except:

* it has twice as many rows and columns, and its boxes are smaller by 50%
* paint and unpaint are not immediately rendered, but rather wait for update


author: Michael Siff year: 2022

"""


import tkinter as _Tk


ROWS = 50
COLUMNS = 80
_SCALE = 15
_BGCOLOR = 'black'
_MONOCOLOR = 'green'
_PAD = 3
_TITLE = 'mini graphics'


# -----------------------------------------------------------------


def paint(x, y):
    """Draws filled box at column x, row y."""
    _open_window()
    if (x, y) not in _g.r:
        e = x*_SCALE + _PAD
        t = y*_SCALE + _PAD
        r = e + _SCALE
        b = t + _SCALE
        _g.r[(x, y)] = _g.canvas.create_rectangle(e, t, r, b, fill=_MONOCOLOR)


def unpaint(x, y):
    """Erases box at column x, row y."""
    _open_window()
    p = (x, y)
    if p in _g.r:
        _g.canvas.delete(_g.r[p])
        del _g.r[p]


def update():
    """Updates screen."""
    _g.root.update()


def clear():
    """Clears graphics window."""
    _open_window()
    _g.r.clear()
    _g.canvas.delete('all')
    _g.root.update()


def _open_window():
    """
    Opens simple graphics window assuming it is not open already.
    The window represents a COLUMNSxROWS grid of "cells"
    where each cell is _SCALEx_SCALE pixels.
    If window already open (and still active), nothing happens.
    """
    global _g
    if _g is None:
        _g = _Graphics()
        _g.root = _Tk.Tk()
        _g.r = {}   # pos->id already drawn rectangles ???
        _g.root.title(_TITLE)
        _g.root.wm_protocol('WM_DELETE_WINDOW', _close_window)
        frame = _Tk.Frame(_g.root)
        frame.pack()
        _g.canvas = _Tk.Canvas(frame,
                               width=COLUMNS*_SCALE + _PAD,
                               height=ROWS*_SCALE + _PAD,
                               bg=_BGCOLOR)
        _g.canvas.pack()
        _g.root.update()


def _close_window():
    """
    Effectively closes graphics window.
    Limitations of Tk makes this not necessarily actually close the window,
    but next call to open_window will close it before opening new window.
    """
    global _g
    if _g:
        _g.root.quit()
        _g.root.destroy()
        _g = None


# ----------------------


class _Graphics:
    pass


_g = None


# -------


import time


def _demo():
    x_base = 0
    for y in range(ROWS):
        for x in range(x_base, COLUMNS, 2):
            paint(x, y)
        x_base = 1 - x_base
    update()
    time.sleep(2)
    x_base = 0
    for y in range(ROWS):
        for x in range(x_base, COLUMNS, 3):
            unpaint(x, y)
            update()
        x_base = 1 - x_base


if __name__ == '__main__':
    _demo()
